<?php
/**
 * This Software is the property of Data Development and is protected
 * by copyright law - it is NOT Freeware.
 *
 * Any unauthorized use of this software without a valid license
 * is a violation of the license agreement and will be prosecuted by
 * civil and criminal law.
 *
 * http://www.shopmodule.com
 *
 * @copyright (C) D3 Data Development (Inh. Thomas Dartsch)
 * @author    D3 Data Development <support@shopmodule.com>
 * @link      http://www.oxidmodule.com
 */

/**
 * Class d3_multivariants_update
 */
class d3_multivariants_update extends d3install_updatebase
{
    public $sModKey = 'd3_multivariants';
    public $sModName = 'mehrdimensionale Varianten';
    public $sModVersion = '3.0.3.0';
    public $sModRevision = '156';
    public $sBaseConf = 'K1zS3RYMS9GMUNHMkdWeTV1aTFkUFhiNENPNnMzOWQ5aGdxTnBHZ1ZxdnY5M3JULzhIbzlhZEpSc2dYb
VBkdkhITWVQVy9EQ2FDeHFLVEZxcFk2eUt6QVBYb2tVNnUwV3hNd2ZicktnSDk3Sm1lN3hQZE8xTVFvb
lMydjJ0RXplejA5S1BCMnRLc2czSW43OWNzR2I2QWxjN0RpKzJFdFVnWnA5dmNaMXB5SXlJWTdjSzdzM
FdOcGM3TTl0RFFkcU5UYmNMZzJQWkpkUHEvZ0dSc25nTmlaNWM4K0hzczU2dmlJdHNJUHhSSzRYNUxMd
HJsaUNyN3piMHNIUGFWYWNQM2xBdUVydk9zK0ZVRStZSTBsNi9PS0tMWDk1Q0FQcDhsc1BEUjd4VGRvd
zg9';
    public $sRequirements = '<?xml version="1.0" encoding="UTF-8"?>
      <jobs>
        <registerModule>
          <module class="article_variant" modulepath="d3/d3_multivariants/modules/controllers/admin/d3_article_variant_mv" />
        </registerModule>
      </jobs>';

    public $sBaseValue = '';

    /* minimale Modul-Connector Version */
    public $sMinModCfgVersion = '4.3.7.0';

    protected $_aUpdateMethods = array(
        array(  // Moduleintrag anlegen, wenn noch nicht vorhanden
            'check' => 'checkModCfgItemExist',
            'do'    => 'updateModCfgItemExist'
        ),
        array(  // exact one whitespace between separator and variantselection
            'check' => 'checkVariantWhitespaces',
            'do'    => 'updateVariantWhitespaces'
        ),
        array(
            'check' => 'hasUnregisteredFiles',
            'do'    => 'showUnregisteredFiles'
        ),
        array(  // Moduleintrag aktualisieren
            'check' => 'checkModCfgSameRevision',
            'do'    => 'updateModCfgSameRevision'
        ),
    );

    protected $_aRefreshMetaModuleIds = array('d3_multivariants');

    public $aOldModuleFiles = array();

    /**********************************************************************/
    /*** individuelle Updateprozesse **************************************/
    /**********************************************************************/
    /**
     * @return bool
     */
    public function checkVariantWhitespaces()
    {
        return $this->_checkUpdateFile('d3/d3_multivariants/setup/d3multiVariants_whitespaceQuerys.php');
    }

    /**
     * @return bool
     */
    public function updateVariantWhitespaces()
    {
        // we dont need this configparam
        $blVariantConfigParam = oxRegistry::getConfig()->getConfigParam('d3multivariantsupdate');

        if (false == empty($blVariantConfigParam)) {
            // delete it if available
            $oDb = oxDb::getDb();
            $sShopId = $oDb->quote($this->getShopId());

            $sQ = " DELETE FROM oxconfig WHERE oxshopid = " . $sShopId //
                . " AND oxvarname = 'd3multivariantsupdate' " //
                . " AND oxmodule = 'multiVariants_3.x'";
            $oDb->execute($sQ);
        }

        $blReturn = $this->_doUpdateFile('d3/d3_multivariants/setup/d3multiVariants_whitespaceQuerys.php');

        return $blReturn;
    }

    /**********************************************************************/
    /*** Modul Eintrag in der Datenbank ***********************************/
    /**********************************************************************/
    /**
     * @return bool
     */
    public function checkModCfgItemExist()
    {
        $blRet = false;
        foreach ($this->getShopList() as $oShop) {
            /** @var $oShop oxshop */
            $aWhere = array(
                'oxmodid'       => $this->sModKey,
                'oxnewrevision' => $this->sModRevision,
                'oxshopid'      => $oShop->getId(),
            );

            $blRet = $this->_checkTableItemNotExist('d3_cfg_mod', $aWhere);

            if ($blRet) {
                return $blRet;
            }
        }

        return $blRet;
    }

    /**
     * @return bool
     */
    public function updateModCfgItemExist()
    {
        $blRet = false;

        if ($this->checkModCfgItemExist()) {
            foreach ($this->getShopList() as $oShop) {
                /** @var $oShop oxshop */
                $aWhere = array(
                    'oxmodid'       => $this->sModKey,
                    'oxshopid'      => $oShop->getId(),
                    'oxnewrevision' => $this->sModRevision,
                );

                if ($this->_checkTableItemNotExist('d3_cfg_mod', $aWhere)) {
                    // update don't use this property
                    unset($aWhere['oxnewrevision']);

                    $aInsertFields = array(
                        'OXID'           => array(
                            'content'      => "md5('" . $this->sModKey . " " . $oShop->getId() . "')",
                            'force_update' => false,
                            'use_quote'    => false,
                        ),
                        'OXSHOPID'       => array(
                            'content'      => $oShop->getId(),
                            'force_update' => false,
                            'use_quote'    => true,
                        ),
                        'OXMODID'        => array(
                            'content'      => $this->sModKey,
                            'force_update' => true,
                            'use_quote'    => true,
                        ),
                        'OXNAME'         => array(
                            'content'      => $this->sModName,
                            'force_update' => true,
                            'use_quote'    => true,
                        ),
                        'OXACTIVE'       => array(
                            'content'      => "0",
                            'force_update' => false,
                            'use_quote'    => false,
                        ),
                        'OXBASECONFIG'   => array(
                            'content'      => $this->sBaseConf,
                            'force_update' => true,
                            'use_quote'    => true,
                        ),
                        'OXINSTALLDATE'  => array(
                            'content'      => "NOW()",
                            'force_update' => true,
                            'use_quote'    => false,
                        ),
                        'OXVERSION'      => array(
                            'content'      => $this->sModVersion,
                            'force_update' => true,
                            'use_quote'    => true,
                        ),
                        'OXSHOPVERSION'  => array(
                            'content'      => oxRegistry::getConfig()->getEdition(),
                            'force_update' => true,
                            'use_quote'    => true,
                        ),
                        'OXREQUIREMENTS' => array(
                            'content'      => $this->sRequirements,
                            'force_update' => true,
                            'use_quote'    => true,
                        ),
                        'OXVALUE'        => array(
                            'content'      => $this->sBaseValue,
                            'force_update' => false,
                            'use_quote'    => true,
                        ),
                        'OXNEWREVISION'  => array(
                            'content'      => $this->sModRevision,
                            'force_update' => true,
                            'use_quote'    => true,
                        ),
                    );
                    $aRet          = $this->_updateTableItem2('d3_cfg_mod', $aInsertFields, $aWhere);
                    $blRet         = $aRet['blRet'];
                    $this->setActionLog('SQL', $aRet['sql'], __METHOD__);
                    $this->setUpdateBreak(false);

                    if ($this->getStepByStepMode()) {
                        break;
                    }
                }
            }
        }

        return $blRet;
    }

    /**
     * @return bool
     */
    public function checkModCfgSameRevision()
    {
        return $this->_checkModCfgSameRevision($this->sModKey);
    }

    /**
     * @return bool
     */
    public function updateModCfgSameRevision()
    {
        $blRet = false;

        if ($this->checkModCfgSameRevision()) {
            $aRet = $this->_updateModCfgSameRevision($this->sModKey);

            $this->setActionLog('SQL', $aRet['sql'], __METHOD__);
            $this->setUpdateBreak(false);
            $blRet = $aRet['blRet'];
        }

        return $blRet;
    }

    /**********************************************************************/
    /*** Datei Verzeichniskontrolle ***************************************/
    /**********************************************************************/
    /**
     * @return bool
     */
    public function hasUnregisteredFiles()
    {
        return $this->_hasUnregisteredFiles($this->sModKey, array('d3FileRegister'));
    }

    /**
     * @return bool
     */
    public function showUnregisteredFiles()
    {
        return $this->_showUnregisteredFiles($this->sModKey, array('d3FileRegister'));
    }
}
